package scripts;

import java.net.*;
import java.text.*;
import jinro.*;
import scripts.roles.*;
import org.schwering.irc.lib.*;

class IRCConnector {
	def static strings = JinroLocalization.getBundle("IRCConnector.properties");
	def server;
	def user;
	def messageLog;
	def MOTDMessage;
	def connection;
	def privmsgClosures;
	def changeTopicClosures;
	def pool;
	IRCConnector() {
		server = [
			host   : "",
			port   : 0,
			pass   : "",
			charset: ""
		];
		user = [
			nick    : "",
			user    : "",
			name    : "",
			mainch  : "",
			channels: [:]
		];
		messageLog = [:];
		MOTDMessage = new StringBuilder();
		privmsgClosures = [];
		changeTopicClosures = [];
	}
	def connect(parameters) {
		if(!connection) {
			server.host    = parameters.host;
			server.port    = parameters.port;
			user.nick      = parameters.nick.replace(" ", "_");
			user.user      = parameters.user?.replace(" ", "_") ?: user.nick;
			user.name      = parameters.name ?: parameters.nick;
			server.pass    = parameters.pass;
			server.charset = parameters.charset ?: "UTF-8";
			user.mainch    = parameters.mainch;
			if(server.host && server.port && server.charset && user.nick && user.mainch) {
				connection = new IRCConnection(server.host, [server.port] as int[], server.pass, user.nick, user.user, user.name);
				connection.daemon = true;
				connection.colors = false;
				connection.pong = true;
				connection.encoding = server.charset;
				connection.timeout = 300000;
				connection.addIRCEventListener(ircEventAdapter);
				try {
					connection.connect();
				} catch(e) {
					connection = null;
				}
				if(!(JinroGame.getProperty("wolvesChat") =~ /^[#&][^ ,]+$/)) {
					JinroGame.setProperty("wolvesChat", user.mainch + "_wolves");
				}
				if(!(JinroGame.getProperty("sharersChat") =~ /^[#&][^ ,]+$/)) {
					JinroGame.setProperty("sharersChat", user.mainch + "_sharers");
				}
				pool = [:];
				Thread.startDaemon(watch);
			} else {
				addLog(strings.getString("error.lackOfParameters"));
			}
		}
	}
	def disconnect() {
		if(connection) {
			connection.doQuit();
			connection.close();
			connection = null;
			user.channels.clear();
		}
	}
	def ircEventAdapter = [
		onRegistered: {
			addLog(strings.getString("info.connected", server.host, server.port));
			user.channels.each { ch, data ->
				join(ch, data.title, true);
			};
			join(user.mainch, strings.getString("mainch"));
		},
		onDisconnected: {
			addLog(strings.getString("info.disconnected", server.host, server.port));
		},
		onError: { a, b ->
			if(a instanceof String) {
				addLog(strings.getString("error.message", a));
			} else {
				addLog(strings.getString("error.numericReply", a, b));
				if(a == 451) {
					disconnect();
				}
			}
		},
		onMode: { ch, usr, modes ->
			if(ch instanceof String) {
				modes.count.times { i ->
					i++;
					def op = modes.getOperatorAt(i) == ('+' as char);
					def mode = modes.getModeAt(i) as String;
					def arg = modes.getArgAt(i);
					user.channels[ch]["ahoqv".contains(mode) ? "userMode" : "channelMode"][mode] = op ? arg ?: op : op;
					if(mode == "f") {
						def limit = modes.line =~ /(?:^| )\[([0-9]+)t\]:([0-9]+)(?: |$)/;
						user.channels[ch].sendDelay = op && limit ? (limit[0][2] as int) * 1000 / (limit[0][1] as int) + 100 : 0;
					}
				};
				addLog(strings.getString("info.mode", ch, modes.line));
			}
		},
		onNotice: { target, usr, msg ->
			addMessageLog(usr.nick, target, msg, true);
		},
		onPrivmsg: { target, usr, msg ->
			addMessageLog(usr.nick, target, msg);
			privmsgClosures.each { it(target, usr.nick, msg) };
		},
		onReply: { num, value, msg ->
			switch(num) {
				case 372:
				case 375..376:
					if(msg) {
						value += " ";
						addMOTDMessage(value.substring(Math.min(user.nick.size() + 1, value.size())) + msg, num == 375);
					}
					break;
				case 1..5:
				case 251..256:
				case 265..266:
					if(msg) {
						value += " ";
						addLog(value.substring(Math.min(user.nick.size() + 1, value.size())) + msg);
					}
					break;
				case 332:
					user.channels[value.split(/ /)[1]].topic = msg;
					changeTopicClosures.each { it(value, msg) };
					break;
				case 324:
					def values = value.split(/ /).toList();
					def modes = new IRCModeParser(values[2], msg);
					modes.count.times { i ->
						i++;
						def op = modes.getOperatorAt(i) == ('+' as char);
						def mode = modes.getModeAt(i) as String;
						def arg = modes.getArgAt(i);
						user.channels[values[1]]["ahoqv".contains(mode) ? "userMode" : "channelMode"][mode] = op ? arg ?: op : op;
						if(mode == "f") {
							def limit = modes.line =~ /(?:^| )\[([0-9]+)t\]:([0-9]+)(?: |$)/;
							user.channels[values[1]].sendDelay = op && limit ? (limit[0][2] as int) * 1000 / (limit[0][1] as int) + 100 : 0;
						}
					};
					addLog(strings.getString("info.mode", values[1], modes.line));
					break;
			}
		},
		onTopic: { ch, usr, topic ->
			user.channels[ch].topic = topic;
			changeTopicClosures.each { it(channel, topic) };
		}
	] as IRCEventAdapter;
	def addMessageLog(nick, target, message, isnotice = false) {
		if(target == user.nick) {
			target = nick;
		}
		if(!messageLog[target]) {
			messageLog[target] = [ read: false, messages: [] ];
		}
		messageLog[target].read = false;
		messageLog[target].messages << [ nick: nick, message: message, day: JinroGame.day, time: JinroGame.time, notice: isnotice ];
	}
	def addMOTDMessage(text, first) {
		synchronized(MOTDMessage) {
			if(first) {
				MOTDMessage.delete(0, MOTDMessage.size());
			}
			MOTDMessage << text << "\n".denormalize();
		}
	}
	def addPrivmsgListener(closure) {
		if(!privmsgClosures.contains(closure)) {
			privmsgClosures << closure;
		}
	}
	def removePrivmsgListener(closure) {
		privmsgClosures.remove(closure);
	}
	def addChangeTopicListener(closure) {
		if(!changeTopicClosures.contains(closure)) {
			changeTopicClosures << closure;
		}
	}
	def removeChangeTopicListener(closure) {
		changeTopicClosures.remove(closure);
	}
	def send(target, message, notice = false) {
		if(connection) {
			if(IRCUtil.isChan(target)) {
				if(pool[target]) {
					pool[target].messages << [ message: message, notice: notice ];
				} else {
					pool[target] = [ messages: [[ message: message, notice: notice ]], time: 0 ];
				}
			} else {
				if(notice) {
					connection.doNotice(target, message);
				} else {
					connection.doPrivmsg(target, message);
				}
				addMessageLog(user.nick, target, message, notice);
			}
			return true;
		}
		return false;
	}
	def join(target, title = "", force = false) {
		if(connection && (!user.channels[target] || force)) {
			user.channels[target] = [ title: title, topic: "", userMode: [:], channelMode: [:], sendDelay: 0 ];
			connection.doJoin(target);
			addLog(strings.getString("info.joinned", target));
			connection.doMode(target);
			connection.doTopic(target);
		}
	}
	def part(target) {
		if(connection && user.channels[target]) {
			user.channels.remove(target);
			connection.doPart(target);
			messageLog.remove(target);
			addLog(strings.getString("info.parted", target));
		}
	}
	def invite(target, channel) {
		connection?.doInvite(target, channel);
	}
	def isChannel(target) {
		return IRCUtil.isChan(target);
	}
	def watch = {
		synchronized(watch) {
			while(connection) {
				synchronized(pool) {
					pool.each { target, pooleddata ->
						if(user.channels[target]) {
							if(user.channels[target].userMode.o) {
								pooleddata.messages.each {
									if(it.notice) {
										connection.doNotice(target, it.message);
									} else {
										connection.doPrivmsg(target, it.message);
									}
								}
								pooleddata.messages.clear();
							} else if(pooleddata.messages && pooleddata.time + user.channels[target].sendDelay < java.lang.System.currentTimeMillis()) {
								def message = pooleddata.messages.remove(0);
								if(message.notice) {
									connection.doNotice(target, message.message);
								} else {
									connection.doPrivmsg(target, message.message);
								}
								addMessageLog(user.nick, target, message.message, message.notice);
								pooleddata.time = java.lang.System.currentTimeMillis();
							}
						}
					};
				}
				sleep(10);
			}
		}
	};
	def addLog(message) {
		JinroGame.addLog(strings.getString("log.format", message));
	}
}