package scripts;

import java.text.Collator;
import jinro.*;
import org.eclipse.swt.custom.*;
import org.eclipse.swt.dnd.*;
import org.eclipse.swt.events.*;
import org.eclipse.swt.graphics.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.*;

class IRCViewer {
	def static strings = JinroLocalization.getBundle("JinroInterface.properties").attachBundle("IRCViewer.properties");
	def static bounds;
	def static chbounds = [:];
	def static talkbounds;
	def mainwindow;
	def motdText;
	def viewing;
	def changed;
	def chwindows;
	def chList;
	def talkwindow;
	def talkLogText;
	def talkList;
	def mainchBtn;
	def joinBtn;
	def talkListValues;
	def IRCViewer() {
		def irc = JinroGame.getProperty("ircConnector");
		chwindows = [:];
		talkListValues = [];
		makeMainWindow();
		changeViewingTalk("");
		def myColor = JinroInterface.display.getSystemColor(SWT.COLOR_DARK_GREEN);
		def othersColor = JinroInterface.display.getSystemColor(SWT.COLOR_DARK_BLUE);
		def timeColor = JinroInterface.display.getSystemColor(SWT.COLOR_DARK_GRAY);
		def viewerThread = {
			if(mainwindow.disposed) {
				return;
			}
			chwindows.each { name, data ->
				if(!irc.user.channels[name]) {
					data.part.enabled = false;
					return;
				} else {
					data.part.enabled = name != irc.user.mainch;
				}
				def window = data.window;
				def topic = data.topic;
				def logText = data.logText;
				if(topic.text != irc.user.channels[name].topic) {
					topic.text = irc.user.channels[name].topic;
					topic.toolTipText = topic.text;
				}
				window.text = strings.getString("chwindow.title", irc.user.channels[name].title, name);
				def log = irc.messageLog[name];
				if(log && logText.lineCount <= log.messages.size()) {
					def end = false;
					if(logText.lineCount - 1 == logText.getLineIndex(logText.size.y)) {
						end = true;
					}
					((logText.lineCount - 1)..<log.messages.size()).each { i ->
						def msg = log.messages[i];
						def datetime = strings.getString("logText.datetime", msg.day, JinroGame.getProperty("timename")[msg.time]);
						def timeStyle = new StyleRange(logText.text.size(), datetime.size(), timeColor, null);
						logText.append(datetime + " ");
						logText.styleRange = timeStyle;
						def nick = strings.getString("logText.nick", msg.nick);
						def nameStyle = new StyleRange(logText.text.size(), nick.size(), msg.nick == irc.user.nick ? myColor : othersColor, null);
						logText.append(nick + " " + msg.message + "\n".denormalize());
						logText.styleRange = nameStyle;
					}
					if(end) {
						logText.topIndex = logText.lineCount;
					}
				}
				log?.read = true;
			}
			if(talkwindow && !talkwindow.disposed) {
				if(changed) {
					talkLogText.text = "";
					changed = false;
				}
				refreshTalkList();
				def log = irc.messageLog[viewing];
				if(log && talkLogText.lineCount <= log.messages.size()) {
					def end = false;
					if(talkLogText.lineCount - 1 == talkLogText.getLineIndex(talkLogText.size.y)) {
						end = true;
					}
					((talkLogText.lineCount - 1)..<log.messages.size()).each { i ->
						def msg = log.messages[i];
						def datetime = strings.getString("logText.datetime", msg.day, JinroGame.getProperty("timename")[msg.time]);
						def timeStyle = new StyleRange(talkLogText.text.size(), datetime.size(), timeColor, null);
						talkLogText.append(datetime + " ");
						talkLogText.styleRange = timeStyle;
						def nick = strings.getString("logText.nick", msg.nick);
						def nameStyle = new StyleRange(talkLogText.text.size(), nick.size(), msg.nick == irc.user.nick ? myColor : othersColor, null);
						talkLogText.append(nick + " " + msg.message + "\n".denormalize());
						talkLogText.styleRange = nameStyle;
					}
					if(end) {
						talkLogText.topIndex = talkLogText.lineCount;
					}
				}
				log?.read = true;
			}
			mainchBtn.enabled = !!irc.connection;
			joinBtn.enabled = !!irc.connection;
			refreshChannelList();
			if(!motdText.text.contentEquals(irc.MOTDMessage)) {
				motdText.text = irc.MOTDMessage.toString();
			}
		} as Thread;
		Thread.start {
			while(!mainwindow.disposed) {
				JinroInterface.display.asyncExec(viewerThread);
				sleep(250);
			}
		};
	}
	def makeMainWindow() {
		def irc = JinroGame.getProperty("ircConnector");
		mainwindow = new Shell(JinroInterface.display);
		mainwindow.images = JinroInterface.mainwindow.images;
		mainwindow.setMinimumSize(250, 150);
		mainwindow.text = strings.getString("viewer.title");
		mainwindow.layout = new GridLayout(1, true);
		mainwindow.layout.marginWidth = 0;
		mainwindow.layout.marginHeight = 0;
		mainwindow.addShellListener([
			shellClosed: {
				bounds = mainwindow.bounds;
			}
		] as ShellAdapter);
		def composite = new Composite(mainwindow, SWT.NONE);
		composite.layoutData = new GridData(GridData.FILL_BOTH);
		composite.layout = new GridLayout(2, false);
		def leftpane = new Composite(composite, SWT.NONE);
		leftpane.layoutData = new GridData(GridData.FILL_VERTICAL);
		leftpane.layout = new GridLayout(1, false);
		leftpane.layout.marginWidth = 0;
		leftpane.layout.marginHeight = 0;
		mainchBtn = new Button(leftpane, SWT.PUSH);
		mainchBtn.layoutData = new GridData(GridData.FILL_HORIZONTAL);
		mainchBtn.text = strings.getString("mainch");
		mainchBtn.addSelectionListener([
			widgetSelected: {
				makeChannelWindow(irc.user.mainch);
			}
		] as SelectionAdapter);
		mainchBtn.enabled = !!irc.connection;
		def talk = new Button(leftpane, SWT.PUSH);
		talk.layoutData = mainchBtn.layoutData;
		talk.text = strings.getString("talk");
		talk.addSelectionListener([
			widgetSelected: {
				makeTalkWindow();
			}
		] as SelectionAdapter);
		chList = new Table(leftpane, SWT.SINGLE | SWT.BORDER | SWT.V_SCROLL | SWT.FULL_SELECTION);
		chList.headerVisible = true;
		chList.layoutData = new GridData(GridData.FILL_VERTICAL);
		chList.layoutData.widthHint = 100;
		TableColumn column = new TableColumn(chList, SWT.LEFT);
		column.text = strings.getString("chlist.title");
		column.width = 100;
		chList.addKeyListener([
			keyReleased: { e ->
				def items = chList.selection;
				if(items) {
					switch(e.character)
					{
						case SWT.CR:
							makeChannelWindow(items[0].data);
							break;
					}
				}
			}
		] as KeyAdapter);
		chList.addMouseListener([
			mouseDoubleClick: {
				def items = chList.selection;
				if(items) {
					makeChannelWindow(items[0].data);
				}
			}
		] as MouseAdapter);
		refreshChannelList();
		def rightpane = new Composite(composite, SWT.NONE);
		rightpane.layoutData = new GridData(GridData.FILL_BOTH);
		rightpane.layout = new GridLayout(1, false);
		rightpane.layout.marginWidth = 0;
		rightpane.layout.marginHeight = 0;
		motdText = new StyledText(rightpane, SWT.BORDER | SWT.V_SCROLL | SWT.WRAP | SWT.READ_ONLY);
		motdText.layoutData = new GridData(GridData.FILL_BOTH);
		motdText.layoutData.widthHint = 250;
		motdText.layoutData.heightHint = 250;
		motdText.background = JinroInterface.display.getSystemColor(SWT.COLOR_WHITE);
		motdText.foreground = JinroInterface.display.getSystemColor(SWT.COLOR_BLACK);
		def cmdarea = new Composite(rightpane, SWT.NONE);
		cmdarea.layoutData = new GridData(GridData.HORIZONTAL_ALIGN_END);
		cmdarea.layout = new GridLayout(1, false);
		cmdarea.layout.marginWidth = 0;
		cmdarea.layout.marginHeight = 0;
		joinBtn = new Button(cmdarea, SWT.PUSH);
		joinBtn.text = strings.getString("join");
		joinBtn.enabled = !!irc.connection;
		joinBtn.addSelectionListener([
			widgetSelected: {
				def dialog = JinroInterface.makeDialog(strings.getString("join"), mainwindow);
				def jcomposite = new Composite(dialog, SWT.NONE);
				jcomposite.layoutData = new GridData(GridData.FILL_BOTH);
				jcomposite.layout = new GridLayout(2, false);
				new Label(jcomposite, SWT.NONE).text = strings.getString("join.title");
				def jtitle = new Text(jcomposite, SWT.BORDER);
				jtitle.layoutData = new GridData(GridData.FILL_HORIZONTAL);
				jtitle.layoutData.widthHint = 150;
				new Label(jcomposite, SWT.NONE).text = strings.getString("join.name");
				def jname = new Text(jcomposite, SWT.BORDER);
				jname.layoutData = jtitle.layoutData;
				def jbutton = JinroInterface.makeButtons([strings.getString("dialog.ok"), strings.getString("dialog.cancel")].toArray(new String[0]), dialog);
				jbutton[0].addSelectionListener([
					widgetSelected: {
						if(!irc.isChannel(jname.text)) {
							def msg = new MessageBox(dialog, SWT.OK | SWT.ICON_WARNING);
							msg.text = strings.getString("join.error");
							msg.message = strings.getString("join.error.invalidName");
							msg.open();
						} else if(irc.user.channels.containsKey(jname.text)) {
							def msg = new MessageBox(dialog, SWT.OK | SWT.ICON_WARNING);
							msg.text = strings.getString("join.error");
							msg.message = strings.getString("join.error.alreadyJoinned");
							msg.open();
						} else if(irc.user.channels.find({ name, channel -> jtitle.text && channel.title == jtitle.text }) != null) {
							def msg = new MessageBox(dialog, SWT.OK | SWT.ICON_WARNING);
							msg.text = strings.getString("join.error");
							msg.message = strings.getString("join.error.duplicatedTitle");
							msg.open();
						} else {
							irc.join(jname.text, jtitle.text);
							dialog.dispose();
						}
					}
				] as SelectionAdapter);
				jbutton[1].addSelectionListener([
					widgetSelected: {
						dialog.dispose();
					}
				] as SelectionAdapter);
				dialog.pack();
				JinroInterface.setLocationToCenterParent(dialog);
				dialog.open();
			}
		] as SelectionAdapter);
		mainwindow.pack();
		if(bounds) {
			mainwindow.bounds = bounds;
		}
		mainwindow.open();
	}
	def makeChannelWindow(name) {
		if(chwindows[name]) {
			chwindows[name].window.setActive();
		} else {
			def irc = JinroGame.getProperty("ircConnector");
			def window = new Shell(mainwindow, SWT.SHELL_TRIM);
			window.images = JinroInterface.mainwindow.images;
			window.setMinimumSize(150, 150);
			window.text = strings.getString("chwindow.title", irc.user.channels[name].title, name);
			window.layoutData = new GridData(GridData.FILL_BOTH);
			window.layout = new GridLayout(1, true);
			window.addDisposeListener([
				widgetDisposed: {
					chbounds[name] = window.bounds;
					chwindows.remove(name);
				}
			] as DisposeListener);
			def topic = new Label(window, SWT.NONE);
			topic.layoutData = new GridData(GridData.FILL_HORIZONTAL);
			topic.text = irc.user.channels[name].topic;
			topic.toolTipText = topic.text;
			def logText = new StyledText(window, SWT.BORDER | SWT.V_SCROLL | SWT.WRAP | SWT.READ_ONLY);
			logText.layoutData = new GridData(GridData.FILL_BOTH);
			logText.background = JinroInterface.display.getSystemColor(SWT.COLOR_WHITE);
			logText.foreground = JinroInterface.display.getSystemColor(SWT.COLOR_BLACK);
			def msgbox = new Text(window, SWT.BORDER);
			msgbox.layoutData = new GridData(GridData.FILL_HORIZONTAL);
			msgbox.addKeyListener([
				keyReleased: { e ->
					switch(e.character)
					{
						case SWT.CR:
							if(irc.connection && irc.user.channels[name] && msgbox.text) {
								if(irc.send(name, msgbox.text)) {
									msgbox.text = "";
								}
							}
					}
				}
			] as KeyAdapter);
			def partctner = new Composite(window, SWT.NONE);
			partctner.layoutData = new GridData(GridData.HORIZONTAL_ALIGN_END);
			partctner.layout = new GridLayout(1, false);
			partctner.layout.marginWidth = 0;
			partctner.layout.marginHeight = 0;
			def part = new Button(partctner, SWT.PUSH);
			part.text = strings.getString("part");
			if(name == irc.user.mainch) {
				part.enabled = false;
			}
			part.addSelectionListener([
				widgetSelected: {
					def dialog = new MessageBox(mainwindow, SWT.OK | SWT.CANCEL | SWT.ICON_WARNING);
					dialog.text = mainwindow.text;
					dialog.message = strings.getString("part.confirm", irc.user.channels[name].title, name);
					if(dialog.open() == SWT.OK) {
						irc.part(name);
					}
				}
			] as SelectionAdapter);
			window.pack();
			if(chbounds[name]) {
				window.bounds = chbounds[name];
			} else {
				window.bounds = new Rectangle(window.bounds.x, window.bounds.y, 350, 300);
			}
			window.open();
			chwindows[name] = [ window: window, logText: logText, topic: topic, part: part ];
		}
	}
	def makeTalkWindow() {
		if(talkwindow && !talkwindow.isDisposed()) {
			talkwindow.setActive();
		} else {
			def irc = JinroGame.getProperty("ircConnector");
			talkwindow = new Shell(mainwindow, SWT.SHELL_TRIM);
			talkwindow.images = JinroInterface.mainwindow.images;
			talkwindow.setMinimumSize(250, 150);
			talkwindow.text = strings.getString("talk");
			talkwindow.layout = new GridLayout(1, true);
			talkwindow.layout.marginWidth = 0;
			talkwindow.layout.marginHeight = 0;
			talkwindow.addDisposeListener([
				widgetDisposed: {
					talkbounds = talkwindow.bounds;
				}
			] as DisposeListener);
			def composite = new Composite(talkwindow, SWT.NONE);
			composite.layoutData = new GridData(GridData.FILL_BOTH);
			composite.layout = new GridLayout(2, false);
			talkList = new Table(composite, SWT.SINGLE | SWT.BORDER | SWT.V_SCROLL | SWT.FULL_SELECTION);
			talkList.headerVisible = true;
			talkList.layoutData = new GridData(GridData.FILL_VERTICAL);
			talkList.layoutData.widthHint = 130;
			TableColumn column = new TableColumn(talkList, SWT.LEFT);
			column.text = strings.getString("name");
			column.width = 130;
			talkList.addSelectionListener([
				widgetSelected: {
					def index = talkList.selectionIndex;
					if(index >= 0) {
						changeViewingTalk(talkListValues[index]);
					}
				}
			] as SelectionAdapter);
			refreshTalkList();
			def rightpane = new Composite(composite, SWT.NONE);
			rightpane.layoutData = new GridData(GridData.FILL_BOTH);
			rightpane.layout = new GridLayout(1, false);
			rightpane.layout.marginWidth = 0;
			rightpane.layout.marginHeight = 0;
			talkLogText = new StyledText(rightpane, SWT.BORDER | SWT.V_SCROLL | SWT.WRAP | SWT.READ_ONLY);
			talkLogText.layoutData = new GridData(GridData.FILL_BOTH);
			talkLogText.layoutData.widthHint = 300;
			talkLogText.layoutData.heightHint = 200;
			talkLogText.background = JinroInterface.display.getSystemColor(SWT.COLOR_WHITE);
			talkLogText.foreground = JinroInterface.display.getSystemColor(SWT.COLOR_BLACK);
			def msgbox = new Text(rightpane, SWT.BORDER);
			msgbox.layoutData = new GridData(GridData.FILL_HORIZONTAL);
			msgbox.addKeyListener([
				keyReleased: { e ->
					switch(e.character)
					{
						case SWT.CR:
							if(irc.connection && viewing && msgbox.text) {
								if(irc.send(viewing, msgbox.text)) {
									msgbox.text = "";
								}
							}
					}
				}
			] as KeyAdapter);
			talkwindow.pack();
			if(talkbounds) {
				talkwindow.bounds = talkbounds;
			}
			talkwindow.open();
		}
	}
	def boldFont;
	def plainFont;
	def italicFont;
	def refreshChannelList() {
		def irc = JinroGame.getProperty("ircConnector");
		def selected = chList.selection;
		if(selected) {
			selected = selected[0].data;
		} else {
			selected = null;
		}
		chList.removeAll();
		irc.user.channels.findAll{ name, channel -> irc.isChannel(name) && name!= irc.user.mainch }.each { name, channel ->
			def item = new TableItem(chList, SWT.NONE);
			item.text = channel.title ?: name;
			item.data = name;
			if(selected == name) {
				chList.selection = item;
			}
			if(!plainFont) {
				def fds = item.font.fontData;
				fds.toList().each { it.style = SWT.NORMAL };
				plainFont = new Font(JinroInterface.display, fds);
			}
			if(!italicFont) {
				def fds = item.font.fontData;
				fds.toList().each { it.style = SWT.ITALIC };
				italicFont = new Font(JinroInterface.display, fds);
			}
			item.font = channel.title ? plainFont : italicFont;
		}
	}
	def refreshTalkList() {
		def irc = JinroGame.getProperty("ircConnector");
		def actors = JinroGame.getActors().findAll { it.skypeid };
		def collator = Collator.getInstance(JinroLocalization.getLocale());
		collator.strength = Collator.PRIMARY;
		actors.sort { one, two ->
			return collator.compare(one.name, two.name);
		}
		talkList.removeAll();
		talkListValues.clear();
		def unreadf = JinroInterface.display.getSystemColor(SWT.COLOR_BLUE);
		def unreadb = JinroInterface.display.getSystemColor(SWT.COLOR_GRAY);
		actors.each {
			def item = new TableItem(talkList, SWT.NONE);
			item.text = it.name;
			talkListValues << it.skypeid;
			if(it.skypeid == viewing) {
				talkList.selection = item;
			} else if(!irc.messageLog[it.skypeid]?.read) {
				item.foreground = unreadf;
				item.background = unreadb;
			}
			if(!boldFont) {
				def fds = item.font.fontData;
				fds.toList().each { it.style = SWT.BOLD };
				boldFont = new Font(JinroInterface.display, fds);
			}
			item.font = boldFont;
		}
		def nicks = irc.messageLog.keySet().toArray().toList().findAll{ !irc.isChannel(it) && !JinroGame.getNamebySkypeid(it) };
		nicks.sort { one, two ->
			return collator.compare(one, two);
		}
		nicks.each {
			def item = new TableItem(talkList, SWT.NONE);
			item.text = it;
			talkListValues << it;
			if(it == viewing) {
				talkList.selection = item;
			} else if(!irc.messageLog[it]?.read) {
				item.foreground = unreadf;
				item.background = unreadb;
			}
			if(!plainFont) {
				def fds = item.font.fontData;
				fds.toList().each { it.style = SWT.NORMAL };
				plainFont = new Font(JinroInterface.display, fds);
			}
			item.font = plainFont;
		}
	}
	def changeViewingTalk(target) {
		if(viewing != target) {
			viewing = target;
			def name = JinroGame.getNamebySkypeid(target);
			talkwindow?.text = name ? strings.getString("talkwindow.title.entered", name, target) : strings.getString("talkwindow.title", target);
			changed = true;
		}
	}
}
