package scripts;

import groovy.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.lang.reflect.*;
import java.text.Collator;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;
import javax.swing.table.*;
import jinro.*;

class IRCViewer {
	def static strings = JinroLocalization.getBundle("JinroInterface.properties").attachBundle("IRCViewer.properties");
	def static bounds;
	def static chbounds = [:];
	def static talkbounds;
	def mainwindow;
	def motdText;
	def viewing;
	def talkCount;
	def changed;
	def chwindows;
	def chList;
	def talkwindow;
	def talkLogText;
	def talkList;
	def mainchBtn;
	def joinBtn;
	def talkListValues;
	def chListValues;
	def IRCViewer() {
		def irc = JinroGame.getProperty("ircConnector");
		talkCount = 0;
		chwindows = [:];
		talkListValues = [];
		chListValues = [];
		makeMainWindow();
		changeViewingTalk("");
		def myColor = new Color(0, 128, 0);
		def othersColor = new Color(0, 0, 128);
		def timeColor = Color.GRAY;
		def viewerThread = {
			if(!mainwindow) {
				return;
			}
			chwindows.each { name, data ->
				if(!irc.user.channels[name]) {
					data.part.enabled = false;
					return;
				} else {
					data.part.enabled = name != irc.user.mainch;
				}
				def window = data.window;
				def topic = data.topic;
				def logText = data.logText;
				if(irc.user.channels[name] && topic.text != irc.user.channels[name].topic) {
					topic.text = irc.user.channels[name].topic;
					topic.toolTipText = topic.text;
				}
				window.title = strings.getString("chwindow.title", irc.user.channels[name]?.title ?: "", name);
				def log = irc.messageLog[name];
				//def count = logText.text.normalize().findAll(/\n/).size();
				if(log && data.count < log.messages.size()) {
					def end = false;
					def scrollbar = logText.parent.parent.verticalScrollBar;
					if(scrollbar.maximum - scrollbar.visibleAmount <= scrollbar.value) {
						end = true;
					}
					def doc = logText.styledDocument;
					(data.count..<log.messages.size()).each { i ->
						def msg = log.messages[i];
						def datetime = strings.getString("logText.datetime", msg.day, JinroGame.getProperty("timename")[msg.time]);
						def timeStyle = new SimpleAttributeSet();
						StyleConstants.setForeground(timeStyle, timeColor);
						doc.insertString(doc.length, datetime + " ", timeStyle);
						def nick = strings.getString("logText.nick", msg.nick);
						def nameStyle = new SimpleAttributeSet();
						StyleConstants.setForeground(nameStyle, msg.nick == irc.user.nick ? myColor : othersColor);
						doc.insertString(doc.length, nick + " ", nameStyle);
						doc.insertString(doc.length, msg.message + "\n".denormalize(), null);
						data.count++;
					}
					if(end) {
						logText.caretPosition = doc.length;
					}
				}
				log?.read = true;
			}
			if(talkwindow) {
				if(changed) {
					talkLogText.text = "";
					changed = false;
				}
				refreshTalkList();
				def log = irc.messageLog[viewing];
				if(log && talkCount < log.messages.size()) {
					def scrollbar = talkLogText.parent.parent.verticalScrollBar;
					def doc = talkLogText.styledDocument;
					//def end = false;
					(talkCount..<log.messages.size()).each { i ->
						def msg = log.messages[i];
						def datetime = strings.getString("logText.datetime", msg.day, JinroGame.getProperty("timename")[msg.time]);
						def timeStyle = new SimpleAttributeSet();
						StyleConstants.setForeground(timeStyle, timeColor);
						doc.insertString(doc.length, datetime + " ", timeStyle);
						def nick = strings.getString("logText.nick", msg.nick);
						def nameStyle = new SimpleAttributeSet();
						StyleConstants.setForeground(nameStyle, msg.nick == irc.user.nick ? myColor : othersColor);
						doc.insertString(doc.length, nick + " ", nameStyle);
						doc.insertString(doc.length, msg.message + "\n".denormalize(), null);
						talkCount++;
					}/*
					if(end) {
						talkLogText.caretPosition = doc.length;
					}*/
				}
				log?.read = true;
			}
			mainchBtn.enabled = !!irc.connection;
			joinBtn.enabled = !!irc.connection;
			refreshChannelList();
			if(!motdText.text.contentEquals(irc.MOTDMessage)) {
				motdText.text = irc.MOTDMessage.toString();
			}
		} as Thread;
		Thread.start {
			while(mainwindow.displayable) {
				SwingUtilities.invokeLater(viewerThread);
				sleep(250);
			}
		};
	}
	def makeMainWindow() {
		def irc = JinroGame.getProperty("ircConnector");
		def swing = new SwingBuilder();
		mainwindow = swing.frame(
			title: strings.getString("viewer.title"),
			iconImages: JinroInterface.mainwindow.iconImages,
			minimumSize: [250, 150],
			size: [350, 250],
			windowClosing: {
				bounds = mainwindow.bounds;
				mainwindow = null;
			}
		) {
			borderLayout()
			panel(constraints: BorderLayout.WEST, preferredSize: [120, 100]) {
				boxLayout(axis: BoxLayout.Y_AXIS)
				mainchBtn = button(text: strings.getString("mainch"), enabled: !!irc.connection, actionPerformed: {
					makeChannelWindow(irc.user.mainch);
				}, maximumSize: [1000, 50], alignmentX: Component.CENTER_ALIGNMENT)
				button(text: strings.getString("talk"), actionPerformed: {
					makeTalkWindow();
				}, maximumSize: [1000, 50], alignmentX: Component.CENTER_ALIGNMENT)
				scrollPane(preferredSize: [120, 100]) {
					chList = table(fillsViewportHeight: true, columnSelectionAllowed: false, rowSelectionAllowed: true, keyPressed: { e ->
						switch(e.keyCode) {
							case KeyEvent.VK_ENTER:
								if(chList.selectedRow >= 0) {
									makeChannelWindow(chListValues[chList.selectedRow]);
								}
								break;
						}
					}, mouseClicked: { e ->
						if(e.clickCount == 2 && chList.rowAtPoint(e.point) >= 0) {
							makeChannelWindow(chListValues[chList.rowAtPoint(e.point)]);
						}
					}) {
						tableModel() {
							propertyColumn(header: strings.getString("chlist.title"), propertyName: "title", preferredWidth: 100)
						}
					}
				}
			}
			panel(constraints: BorderLayout.CENTER) {
				boxLayout(axis: BoxLayout.Y_AXIS)
				panel(preferredSize: [250, 250]) {
					borderLayout()
					emptyBorder(top: 2, right: 2, bottom: 0, left: 2, parent: true)
					scrollPane(verticalScrollBarPolicy: ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS) {
						motdText = textPane(editable: false, alignmentX: Component.CENTER_ALIGNMENT, foreground: Color.BLACK, background: Color.WHITE)
					}
				}
				panel() {
					flowLayout(alignment: FlowLayout.TRAILING)
					joinBtn = button(text: strings.getString("join"), enabled: !!irc.connection, actionPerformed: {
						def jtitle;
						def jname;
						def size = [150, swing.textField().preferredSize.height as int];
						def gbcp = [fill: GridBagConstraints.HORIZONTAL, insets: [3, 3, 3, 3]];
						def dialog;
						def buttons = [
							{ swing.button(text: strings.getString("dialog.ok"), actionPerformed: {
								if(!irc.isChannel(jname.text)) {
									JOptionPane.showMessageDialog(dialog, strings.getString("join.error.invalidName"), strings.getString("join.error"), JOptionPane.ERROR_MESSAGE);
								} else if(irc.user.channels[jname.text]) {
									JOptionPane.showMessageDialog(dialog, strings.getString("join.error.alreadyJoinned"), strings.getString("join.error"), JOptionPane.ERROR_MESSAGE);
								} else if(irc.user.channels.find({ name, channel -> jtitle.text && channel.title == jtitle.text }) != null) {
									JOptionPane.showMessageDialog(dialog, strings.getString("join.error.duplicatedTitle"), strings.getString("join.error"), JOptionPane.ERROR_MESSAGE);
								} else {
									irc.join(jname.text, jtitle.text);
									dialog.dispose();
								}
								dialog.dispose();
							}) },
							{ swing.button(text: strings.getString("dialog.cancel"), actionPerformed: {
								dialog.dispose();
							}) }
						];
						dialog = swing.dialog(owner: mainwindow, title: strings.getString("join"), pack: true, resizable: false) {
							boxLayout(axis: BoxLayout.Y_AXIS)
							panel() {
								gridBagLayout()
								label(text: strings.getString("join.title"), constraints: gbc(gbcp))
								jtitle = textField(preferredSize: size, constraints: gbc(gbcp))
								label(text: strings.getString("join.name"), constraints: gbc(gbcp << [gridy: 1]))
								jname = textField(constraints: gbc(gbcp))
							}
							panel() {
								flowLayout(alignment: FlowLayout.TRAILING)
								def btns = [];
								if(JinroInterface.isMac || JinroInterface.isGnome) {
									btns[1] = buttons[1]()
									btns[0] = buttons[0]()
								} else {
									btns[0] = buttons[0]()
									btns[1] = buttons[1]()
								}
								def btnsize = [Math.max(btns[0].preferredSize.width, btns[1].preferredSize.width) as int, btns[0].preferredSize.height as int];
								btns.each {
									it.preferredSize = btnsize;
								};
								btns[0].rootPane.defaultButton = btns[0];
							}
						}
						dialog.locationRelativeTo = mainwindow;
						dialog.visible = true;
					})
				}
			}
		}
		chList.setDefaultRenderer(Object.class, new ChannelListRenderer());
		chList.setDefaultEditor(Object.class, null);
		refreshChannelList();
		if(bounds) {
			mainwindow.bounds = bounds;
		}
		mainwindow.visible = true;
	}
	def makeChannelWindow(name) {
		if(chwindows[name]) {
			chwindows[name].window.visible = true;
		} else {
			def irc = JinroGame.getProperty("ircConnector");
			def swing = new SwingBuilder();
			def window;
			def topic;
			def logText;
			def msgbox;
			def part;
			def chdata = irc.user.channels[name];
			window = swing.dialog(
				owner: mainwindow,
				title: strings.getString("chwindow.title", chdata?.title ?: "", name),
				iconImages: JinroInterface.mainwindow.iconImages,
				minimumSize: [150, 150],
				pack: true,
				resizable: true,
				windowClosing: {
					chbounds[name] = window.bounds;
					chwindows.remove(name);
				}
			) {
				borderLayout(hgap: 3, vgap: 3)
				topic = label(constraints: BorderLayout.NORTH, text: chdata?.topic ?: "", toolTipText: chdata?.topic ?: "")
				scrollPane(constraints: BorderLayout.CENTER, preferredSize: [250, 250], verticalScrollBarPolicy: ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS) {
					logText = textPane(editable: false, alignmentX: Component.CENTER_ALIGNMENT, foreground: Color.BLACK, background: Color.WHITE)
				}
				panel(constraints: BorderLayout.SOUTH) {
					boxLayout(axis: BoxLayout.Y_AXIS)
					msgbox = textField(keyPressed: { e->
						switch(e.keyCode) {
							case KeyEvent.VK_ENTER:
								if(irc.connection && chdata && msgbox.text) {
									if(irc.send(name, msgbox.text)) {
										msgbox.text = "";
									}
								}
								break;
						}
					})
					panel() {
						flowLayout(alignment: FlowLayout.TRAILING)
						part = button(text: strings.getString("part"), enabled: name != irc.user.mainch, actionPerformed: {
							if(JOptionPane.showConfirmDialog(window, strings.getString("join.error.duplicatedTitle"), strings.getString("join.error"), JOptionPane.OK_CANCEL_OPTION) == JOptionPane.OK_OPTION) {
								irc.part(name);
							}
						})
					}
				}
			}
			if(chbounds[name]) {
				window.bounds = chbounds[name];
			} else {
				window.setSize(350, 300);
			}
			window.visible = true;
			chwindows[name] = [ window: window, logText: logText, topic: topic, part: part, count: 0 ];
		}
	}
	def makeTalkWindow() {
		if(talkwindow) {
			talkwindow.visible = true;
		} else {
			def irc = JinroGame.getProperty("ircConnector");
			def swing = new SwingBuilder();
			talkwindow = swing.dialog(
				owner: mainwindow,
				title: strings.getString("talk"),
				iconImages: JinroInterface.mainwindow.iconImages,
				pack: true,
				resizable: true,
				minimumSize: [250, 150],
				windowClosing: {
					talkbounds = talkwindow.bounds;
					talkwindow = null;
				}
			) {
				borderLayout()
				scrollPane(preferredSize: [130, 100], constraints: BorderLayout.WEST) {
					talkList = table(fillsViewportHeight: true, columnSelectionAllowed: false, rowSelectionAllowed: true) {
						tableModel() {
							propertyColumn(header: strings.getString("name"), propertyName: "name", preferredWidth: 100)
						}
					}
				}
				panel(constraints: BorderLayout.CENTER) {
					boxLayout(axis: BoxLayout.Y_AXIS)
					scrollPane(preferredSize: [250, 250], verticalScrollBarPolicy: ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS) {
						talkLogText = textPane(editable: false, alignmentX: Component.CENTER_ALIGNMENT, foreground: Color.BLACK, background: Color.WHITE)
					}
					msgbox = textField(maximumSize: [10000, 50], keyPressed: { e->
						switch(e.keyCode) {
							case KeyEvent.VK_ENTER:
								if(irc.connection && viewing && msgbox.text) {
									if(irc.send(viewing, msgbox.text)) {
										msgbox.text = "";
									}
								}
								break;
						}
					})
				}
			}
			talkList.selectionModel.addListSelectionListener([
				valueChanged: { e ->
					def index = talkList.selectedRow;
					if(index >= 0) {
						changeViewingTalk(talkListValues[index]);
					}
				}
			] as ListSelectionListener);
			talkList.selectionModel.selectionMode = ListSelectionModel.SINGLE_SELECTION;
			talkList.setDefaultRenderer(Object.class, new TalkListRenderer());
			talkList.setDefaultEditor(Object.class, null);
			refreshTalkList();
			if(talkbounds) {
				talkwindow.bounds = talkbounds;
			}
			talkwindow.visible = true;
		}
	}
	def refreshChannelList() {
		def irc = JinroGame.getProperty("ircConnector");
		def rows = [];
		def hasTitles = [];
		chListValues.clear();
		irc.user.channels.findAll{ name, channel -> irc.isChannel(name) && name != irc.user.mainch }.each { name, channel ->
			rows << [title: channel.title ?: name];
			chListValues << name;
			hasTitles << !!channel.title;
		}
		def renderer = chList.getDefaultRenderer(Object.class);
		if(chList.model.rowsModel.value != rows || renderer.hasTitles != hasTitles) {
			renderer.hasTitles = hasTitles;
			chList.model.rowsModel.value = rows;
			chList.model.fireTableDataChanged();
		}
	}
	def refreshTalkList() {
		def irc = JinroGame.getProperty("ircConnector");
		def actors = JinroGame.getActors().findAll { !!it.skypeid };
		def collator = Collator.getInstance(JinroLocalization.getLocale());
		collator.strength = Collator.PRIMARY;
		actors.sort { one, two ->
			return collator.compare(one.name, two.name);
		}
		def unreads = [];
		def rows = [];
		def selected = -1;
		talkListValues.clear();
		actors.each {
			if(it.skypeid == viewing) {
				selected = rows.size();
				unreads << false;
			} else if(!irc.messageLog[it.skypeid]?.read) {
				unreads << true;
			}
			rows << [name: it.name];
			talkListValues << it.skypeid;
		}
		def nicks = irc.messageLog.keySet().toArray().toList().findAll{ !irc.isChannel(it) && !JinroGame.getNamebySkypeid(it) };
		nicks.sort { one, two ->
			return collator.compare(one, two);
		}
		nicks.each {
			if(it == viewing) {
				selected = rows.size();
				unreads << false;
			} else if(!irc.messageLog[it]?.read) {
				unreads << true;
			}
			rows << [name: it];
			talkListValues << it;
		}
		def renderer = talkList.getDefaultRenderer(Object.class);
		if(talkList.model.rowsModel.value != rows) {
			talkList.model.rowsModel.value = rows;
			talkList.model.fireTableDataChanged();
			talkList.selectionModel.setSelectionInterval(selected, selected);
		} else if(renderer.unreads != unreads || renderer.actorsCount != actors.size()) {
			talkList.model.fireTableDataChanged();
			talkList.selectionModel.setSelectionInterval(selected, selected);
		}
		renderer.unreads = unreads;
		renderer.actorsCount = actors.size();
	}
	def changeViewingTalk(target) {
		if(viewing != target) {
			viewing = target;
			talkCount = 0;
			def name = JinroGame.getNamebySkypeid(target);
			talkwindow?.title = name ? strings.getString("talkwindow.title.entered", name, target) : strings.getString("talkwindow.title", target);
			changed = true;
		}
	}
}

class ChannelListRenderer extends DefaultTableCellRenderer {
	def hasTitles;
	def plainFont;
	def italicFont;
	ChannelListRenderer() {
		hasTitles = [];
	}
	Component getTableCellRendererComponent(JTable table, Object value, boolean selected, boolean hasFocus, int row, int column) {
		super.getTableCellRendererComponent(table, value, selected, hasFocus, row, column);
		/*if(selected) {
			setForeground(table.getSelectionForeground());
			setBackground(table.getSelectionBackground());
		} else {
			setForeground(unreads[row] ? Color.BLUE : table.getForeground());
			setBackground(unreads[row] ? Color.LIGHT_GRAY : table.getBackground());
		}*/
		setFont(hasTitles[row] ? plainFont ?: (plainFont = getFont().deriveFont(Font.PLAIN)) : italicFont ?: (italicFont = getFont().deriveFont(Font.ITALIC)));
		return this;
	}
}
class TalkListRenderer extends DefaultTableCellRenderer {
	def unreads;
	def actorsCount;
	def plainFont;
	def boldFont;
	TalkListRenderer() {
		unreads = [];
		actorsCount = 0;
	}
	Component getTableCellRendererComponent(JTable table, Object value, boolean selected, boolean hasFocus, int row, int column) {
		super.getTableCellRendererComponent(table, value, selected, hasFocus, row, column);
		if(selected) {
			setForeground(table.getSelectionForeground());
			setBackground(table.getSelectionBackground());
		} else {
			setForeground(unreads[row] ? Color.BLUE : table.getForeground());
			setBackground(unreads[row] ? Color.LIGHT_GRAY : table.getBackground());
		}
		setFont(row < actorsCount ? boldFont ?: (boldFont = getFont().deriveFont(Font.BOLD)) : plainFont ?: (plainFont = getFont().deriveFont(Font.PLAIN)));
		return this;
	}
}