package scripts.roles;

import scripts.*;
import jinro.*;

// Role: Civilian
// Camp: Civilians
// New role must have this class as an ancestor.

class Civilian {
	def static order = 0;		// All other roles must set larger value than this.
	def static strings = JinroLocalization.getBundle("roles/Civilian.properties");
	def static number = 0;		// All roles must define own "number" property.
	def static bite = false;	// All roles must define own "bite" property.
	// Returns the proper number.
	def static getJustnumber(number) {
		return 0;
	}
	def static isInvisible() {
		return false;
	}
	def getPriority() {
		return 0;
	}
	// Update priority
	// Returns under 0: target -> it
	// Returns over 0 : it -> target
	def getUpdatePriority(target) {
		return 0;
	}
	def actor;
	def Civilian(actor = null) {
		this.actor = actor;
	}
	def getRoleName() {
		return strings.getString("civilian");
	}
	def roleIs(role) {
		return getRoleName() == role.getRoleName();
	}
	def reset() {
		target = null;
		defended = false;
		divined = false;
	}
	def receiveMessage(type, target) {
		// 0: daytime, 1: nighttime
		switch(JinroGame.time) {
			case 0:
				if(type != strings.getString("command.vote")) {
					return;
				}
				if(this.target) {
					return;
				}
				if(target?.name != name && target?.isLive()) {
					this.target = target;
					notice(strings.getString("notice.vote.target", target.name));
				}
				break;
			case 1:
				break;
		}
	}
	// Calls when start argument/night time.
	def startPhase() {
	}
	// Returns the actor's command list
	def getCommands(actors) {
		def commands = [];
		if(!isLive()) {
			return commands;
		}
		// 0: daytime, 1: nighttime
		switch(JinroGame.time) {
			case 0:
				if(!target) {
					actors.findAll{ it.name != name && it.isLive() }.each {
						commands << strings.getString("command.vote") + "@" + it.name;
					}
				}
				break;
			case 1:
				break;
		}
		return commands;
	}
	// Checks if the game can go to next phase.
	def checkStep() {
		// 0: daytime, 1: nighttime
		switch(JinroGame.time) {
			case 0:
				if(target) {
					return true;
				}
				break;
			case 1:
				return true;
		}
		return false;
	}
	// Goes to next phase.
	def update() {
		if(countdown > 0) {
			countdown--;
			if(countdown == 0) {
				killed(nextstate);
			}
		}
	}
	// Checks if the actor is living.
	def isLive() {
		return state.priority > 50;
	}
	// When the actor is killed.
	def killed(state) {
		this.state = state;
		JinroGame.addHistory(this);
		if(state != State.executed) {
			noticeKilled(strings.getString("notice.killed.normal", name));
		}
	}
	def noticeKilled(message) {
		JinroGame.invokeMethod("noticeKilled", message);
	}
	// If it returns true, the role count as a wolf.
	def isWolf() {
		return false;
	}
	// If it returns true, the role count as a civilian.
	def isCivilian() {
		return true;
	}
	// Notifies the actor of the role.
	def noticeRole() {
		notice(strings.getString("notice.role", JinroGame.getAlias(getRoleName())));
	}
	// Judges victory or defeat.
	// if the result is... 0: civilians win, 1: wolves win, 2: foxes win
	def judge(result) {
		return result == 0;
	}
	// Notifies the actor of the message.
	def notice(message) {
		noticelog << message;
		if(skypeid) {
			JinroGame.getProperty("ircConnector").send(skypeid, message);
		}
		JinroGame.addLog(strings.getString("notice.log", name, message));
	}
	// Copy the latest notified message into clipboard.
	def copynoticelog() {
		if(noticelog) {
			JinroInterface.copyClipboard(noticelog[-1]);
		}
	}
	def propertyMissing(String name) {
		return actor.getAt(name);
	}
	def propertyMissing(String name, value) {
		actor.putAt(name, value);
	}
}
