package scripts.roles;

import scripts.*;
import jinro.*;

// Role: Mini fox
// Camp: Foxes

class Minifox extends Fox {
	def static order = 15;
	def static strings = Fox.strings.attachBundle("roles/Minifox.properties");
	def static number = 0;		// All roles must define own "number" property.
	def static bite = false;	// All roles must define own "bite" property.
	def static isInvisible() {
		return false;
	}
	// Returns the proper number.
	def static getJustnumber(number) {
		return 0;
	}
	def getPriority() {
		return 60;
	}
	def getRoleName() {
		return strings.getString("minifox");
	}
	// Notifies the actor of the role.
	def noticeRole() {
		super.noticeRole();
		def actors = JinroGame.getActors();
		actors.findAll{ it.roleIs(new Fox()) }.each {
			notice(strings.getString("notice.fox", it.name, JinroGame.getAlias(it.getRoleName())));
		};
	}
	// Calls when start argument/night time.
	def startPhase() {
		super.startPhase()
		switch(JinroGame.time) {
			case 0:
				break;
			case 1:
				notice(strings.getString("notice.divine"));
				break;
		}
	}
	// Returns the actor's command list
	def getCommands(actors) {
		def commands = super.getCommands(actors);
		if(!isLive()) {
			return commands;
		}
		// 0: daytime, 1: nighttime
		switch(JinroGame.time) {
			case 1:
				if(!target) {
					actors.findAll{ it.name != name && it.isLive() }.each {
						commands << strings.getString("command.divine") + "@" + it.name;
					};
				}
				break;
		}
		return commands;
	}
	def receiveMessage(type, target) {
		super.receiveMessage(type, target)
		switch(JinroGame.time) {
			case 0:
				break;
			case 1:
				if(type != strings.getString("command.divine")) {
					return;
				}
				if(this.target) {
					return;
				}
				if(target?.name != name && target?.isLive()) {
					this.target = target;
					notice(strings.getString("notice.divine.target", target.name));
					divine();
				}
				break;
		}

	}
	// Checks if the game can go to next phase.
	def checkStep() {
		if(!super.checkStep()) {
			return false;
		}
		// 0: daytime, 1: nighttime
		switch(JinroGame.time) {
			case 0:
				return true;
			case 1:
				if(target) {
					return true;
				}
				break;
		}
		return false;
	}
	def divine() {
		if(Math.random() < 0.5) {
			if(target.isWolf()) {
				notice(strings.getString("notice.divine.result", target.name, strings.getString("notice.divine.result.black")));
			} else {
				notice(strings.getString("notice.divine.result", target.name, strings.getString("notice.divine.result.white")));
			}
		} else {
			notice(strings.getString("notice.divine.failed", target.name));
		}
	}
	// Goes to next phase.
	def update() {
		if(countdown > 0) {
			countdown--;
			if(countdown == 0) {
				killed(nextstate);
			}
		}
	}
	// When the actor is killed.
	def killed(state) {
		this.state = state;
		JinroGame.addHistory(this);
		if(state != State.executed) {
			noticeKilled(strings.getString("notice.killed.normal", name));
		}
	}
}